package javax.swing;

import javax.accessibility.AccessibleRole;
import javax.accessibility.AccessibleSelection;
import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;
import javax.swing.event.ChangeEvent;
import java.io.Serializable;
import javax.accessibility.AccessibleContext;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.util.Vector;
import java.awt.event.KeyEvent;
import java.awt.ComponentOrientation;
import javax.swing.event.MenuListener;
import java.beans.PropertyChangeListener;
import java.awt.Component;
import java.awt.Insets;
import java.awt.GraphicsDevice;
import java.awt.GraphicsConfiguration;
import java.awt.Dimension;
import java.awt.GraphicsEnvironment;
import java.awt.Rectangle;
import java.awt.Toolkit;
import javax.swing.plaf.PopupMenuUI;
import javax.swing.plaf.MenuItemUI;
import java.awt.Point;
import javax.swing.event.MenuEvent;
import javax.swing.event.ChangeListener;
import javax.accessibility.Accessible;

public class JMenu extends JMenuItem implements Accessible, MenuElement
{
    private static final String uiClassID = "MenuUI";
    private JPopupMenu popupMenu;
    private ChangeListener menuChangeListener;
    private MenuEvent menuEvent;
    private int delay;
    private Point customMenuLocation;
    private static final boolean TRACE = false;
    private static final boolean VERBOSE = false;
    private static final boolean DEBUG = false;
    protected JMenu.WinListener popupListener;
    
    public JMenu() {
        this("");
    }
    
    public JMenu(String s) {
        super(s);
        this.menuChangeListener = null;
        this.menuEvent = null;
        this.customMenuLocation = null;
    }
    
    public JMenu(Action action) {
        this();
        this.setAction(action);
    }
    
    public JMenu(String s, boolean b) {
        this(s);
    }
    
    @Override
    void initFocusability() {
    }
    
    @Override
    public void updateUI() {
        this.setUI((MenuItemUI)UIManager.getUI(this));
        if (this.popupMenu != null) {
            this.popupMenu.setUI((PopupMenuUI)UIManager.getUI(this.popupMenu));
        }
    }
    
    @Override
    public String getUIClassID() {
        return "MenuUI";
    }
    
    @Override
    public void setModel(ButtonModel buttonModel) {
        ButtonModel model = this.getModel();
        super.setModel(buttonModel);
        if (model != null && this.menuChangeListener != null) {
            model.removeChangeListener(this.menuChangeListener);
            this.menuChangeListener = null;
        }
        if ((this.model = buttonModel) != null) {
            buttonModel.addChangeListener(this.menuChangeListener = this.createMenuChangeListener());
        }
    }
    
    @Override
    public boolean isSelected() {
        return this.getModel().isSelected();
    }
    
    @Override
    public void setSelected(boolean selected) {
        ButtonModel model = this.getModel();
        model.isSelected();
        if (selected != model.isSelected()) {
            this.getModel().setSelected(selected);
        }
    }
    
    public boolean isPopupMenuVisible() {
        this.ensurePopupMenuCreated();
        return this.popupMenu.isVisible();
    }
    
    public void setPopupMenuVisible(boolean b) {
        if (b != this.isPopupMenuVisible() && (this.isEnabled() || !b)) {
            this.ensurePopupMenuCreated();
            if (b && this.isShowing()) {
                Point point = this.getCustomMenuLocation();
                if (point == null) {
                    point = this.getPopupMenuOrigin();
                }
                this.getPopupMenu().show(this, point.x, point.y);
            }
            else {
                this.getPopupMenu().setVisible(false);
            }
        }
    }
    
    protected Point getPopupMenuOrigin() {
        JPopupMenu popupMenu = this.getPopupMenu();
        Dimension size = this.getSize();
        Dimension dimension = popupMenu.getSize();
        if (dimension.width == 0) {
            dimension = popupMenu.getPreferredSize();
        }
        Point locationOnScreen = this.getLocationOnScreen();
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        GraphicsConfiguration graphicsConfiguration = this.getGraphicsConfiguration();
        Rectangle bounds = new Rectangle(defaultToolkit.getScreenSize());
        GraphicsDevice[] screenDevices = GraphicsEnvironment.getLocalGraphicsEnvironment().getScreenDevices();
        for (int i = 0; i < screenDevices.length; ++i) {
            if (screenDevices[i].getType() == 0) {
                GraphicsConfiguration defaultConfiguration = screenDevices[i].getDefaultConfiguration();
                if (defaultConfiguration.getBounds().contains(locationOnScreen)) {
                    graphicsConfiguration = defaultConfiguration;
                    break;
                }
            }
        }
        if (graphicsConfiguration != null) {
            bounds = graphicsConfiguration.getBounds();
            Insets screenInsets = defaultToolkit.getScreenInsets(graphicsConfiguration);
            bounds.width -= Math.abs(screenInsets.left + screenInsets.right);
            bounds.height -= Math.abs(screenInsets.top + screenInsets.bottom);
            locationOnScreen.x -= Math.abs(screenInsets.left);
            locationOnScreen.y -= Math.abs(screenInsets.top);
        }
        int x;
        int int3;
        if ((this.getParent() instanceof JPopupMenu)) {
            int int4 = UIManager.getInt("Menu.submenuPopupOffsetX");
            int3 = UIManager.getInt("Menu.submenuPopupOffsetY");
            if (SwingUtilities.isLeftToRight(this)) {
                x = size.width + int4;
                if (locationOnScreen.x + x + dimension.width >= bounds.width + bounds.x && bounds.width - size.width < 2 * (locationOnScreen.x - bounds.x)) {
                    x = 0 - int4 - dimension.width;
                }
            }
            else {
                x = 0 - int4 - dimension.width;
                if (locationOnScreen.x + x < bounds.x && bounds.width - size.width > 2 * (locationOnScreen.x - bounds.x)) {
                    x = size.width + int4;
                }
            }
            if (locationOnScreen.y + int3 + dimension.height >= bounds.height + bounds.y && bounds.height - size.height < 2 * (locationOnScreen.y - bounds.y)) {
                int3 = size.height - int3 - dimension.height;
            }
        }
        else {
            int int1 = UIManager.getInt("Menu.menuPopupOffsetX");
            int int2 = UIManager.getInt("Menu.menuPopupOffsetY");
            if (SwingUtilities.isLeftToRight(this)) {
                x = int1;
                if (locationOnScreen.x + x + dimension.width >= bounds.width + bounds.x && bounds.width - size.width < 2 * (locationOnScreen.x - bounds.x)) {
                    x = size.width - int1 - dimension.width;
                }
            }
            else {
                x = size.width - int1 - dimension.width;
                if (locationOnScreen.x + x < bounds.x && bounds.width - size.width > 2 * (locationOnScreen.x - bounds.x)) {
                    x = int1;
                }
            }
            int3 = size.height + int2;
            if (locationOnScreen.y + int3 + dimension.height >= bounds.height + bounds.y && bounds.height - size.height < 2 * (locationOnScreen.y - bounds.y)) {
                int3 = 0 - int2 - dimension.height;
            }
        }
        return new Point(x, int3);
    }
    
    public int getDelay() {
        return this.delay;
    }
    
    public void setDelay(int delay) {
        if (delay < 0) {
            throw new IllegalArgumentException("Delay must be a positive integer");
        }
        this.delay = delay;
        return;
    }
    
    private void ensurePopupMenuCreated() {
        if (this.popupMenu == null) {
            (this.popupMenu = new JPopupMenu()).setInvoker(this);
            this.popupListener = this.createWinListener(this.popupMenu);
        }
    }
    
    private Point getCustomMenuLocation() {
        return this.customMenuLocation;
    }
    
    public void setMenuLocation(int x, int y) {
        this.customMenuLocation = new Point(x, y);
        if (this.popupMenu != null) {
            this.popupMenu.setLocation(x, y);
        }
    }
    
    public JMenuItem add(JMenuItem menuItem) {
        this.ensurePopupMenuCreated();
        return this.popupMenu.add(menuItem);
    }
    
    @Override
    public Component add(Component comp) {
        this.ensurePopupMenuCreated();
        this.popupMenu.add(comp);
        return comp;
    }
    
    @Override
    public Component add(Component comp, int index) {
        this.ensurePopupMenuCreated();
        this.popupMenu.add(comp, index);
        return comp;
    }
    
    public JMenuItem add(String s) {
        return this.add(new JMenuItem(s));
    }
    
    public JMenuItem add(Action action) {
        JMenuItem actionComponent = this.createActionComponent(action);
        actionComponent.setAction(action);
        this.add(actionComponent);
        return actionComponent;
    }
    
    protected JMenuItem createActionComponent(Action action) {
        JMenuItem menuItem = new JMenuItem() {
            @Override
            protected PropertyChangeListener createActionPropertyChangeListener(Action action) {
                PropertyChangeListener propertyChangeListener = JMenu.this.createActionChangeListener(this);
                if (propertyChangeListener == null) {
                    propertyChangeListener = super.createActionPropertyChangeListener(action);
                    return propertyChangeListener;
                }
                return propertyChangeListener;
            }
        };
        menuItem.setHorizontalTextPosition(11);
        menuItem.setVerticalTextPosition(0);
        return menuItem;
    }
    
    protected PropertyChangeListener createActionChangeListener(JMenuItem menuItem) {
        return menuItem.createActionPropertyChangeListener0(menuItem.getAction());
    }
    
    public void addSeparator() {
        this.ensurePopupMenuCreated();
        this.popupMenu.addSeparator();
    }
    
    public void insert(String s, int n) {
        if (n < 0) {
            throw new IllegalArgumentException("index less than zero.");
        }
        this.ensurePopupMenuCreated();
        this.popupMenu.insert(new JMenuItem(s), n);
        return;
    }
    
    public JMenuItem insert(JMenuItem menuItem, int n) {
        if (n < 0) {
            throw new IllegalArgumentException("index less than zero.");
        }
        this.ensurePopupMenuCreated();
        this.popupMenu.insert(menuItem, n);
        return menuItem;
    }
    
    public JMenuItem insert(Action action, int n) {
        if (n < 0) {
            throw new IllegalArgumentException("index less than zero.");
        }
        this.ensurePopupMenuCreated();
        JMenuItem menuItem = new JMenuItem(action);
        menuItem.setHorizontalTextPosition(11);
        menuItem.setVerticalTextPosition(0);
        this.popupMenu.insert(menuItem, n);
        return menuItem;
    }
    
    public void insertSeparator(int n) {
        if (n < 0) {
            throw new IllegalArgumentException("index less than zero.");
        }
        this.ensurePopupMenuCreated();
        this.popupMenu.insert(new JPopupMenu.Separator(), n);
        return;
    }
    
    public JMenuItem getItem(int n) {
        if (n < 0) {
            throw new IllegalArgumentException("index less than zero.");
        }
        Component menuComponent = this.getMenuComponent(n);
        if ((menuComponent instanceof JMenuItem)) {
            return (JMenuItem)menuComponent;
        }
        return null;
    }
    
    public int getItemCount() {
        return this.getMenuComponentCount();
    }
    
    public boolean isTearOff() {
        throw new Error("boolean isTearOff() {} not yet implemented");
    }
    
    public void remove(JMenuItem comp) {
        if (this.popupMenu != null) {
            this.popupMenu.remove(comp);
        }
    }
    
    @Override
    public void remove(int n) {
        if (n < 0) {
            throw new IllegalArgumentException("index less than zero.");
        }
        if (n > this.getItemCount()) {
            throw new IllegalArgumentException("index greater than the number of items.");
        }
        if (this.popupMenu != null) {
            this.popupMenu.remove(n);
        }
        return;
    }
    
    @Override
    public void remove(Component comp) {
        if (this.popupMenu != null) {
            this.popupMenu.remove(comp);
        }
    }
    
    @Override
    public void removeAll() {
        if (this.popupMenu != null) {
            this.popupMenu.removeAll();
        }
    }
    
    public int getMenuComponentCount() {
        int componentCount = 0;
        if (this.popupMenu != null) {
            componentCount = this.popupMenu.getComponentCount();
        }
        return componentCount;
    }
    
    public Component getMenuComponent(int n) {
        if (this.popupMenu != null) {
            return this.popupMenu.getComponent(n);
        }
        return null;
    }
    
    public Component[] getMenuComponents() {
        if (this.popupMenu != null) {
            return this.popupMenu.getComponents();
        }
        return new Component[0];
    }
    
    public boolean isTopLevelMenu() {
        return this.getParent() instanceof JMenuBar;
    }
    
    public boolean isMenuComponent(Component component) {
        if (component == this) {
            return true;
        }
        if (component instanceof JPopupMenu && (JPopupMenu)component == this.getPopupMenu()) {
            return true;
        }
        int menuComponentCount = this.getMenuComponentCount();
        Component[] menuComponents = this.getMenuComponents();
        int i = 0;
        while (i < menuComponentCount) {
            Component component2 = menuComponents[i];
            if (component2 == component) {
                return true;
            }
            if (component2 instanceof JMenu && ((JMenu)component2).isMenuComponent(component)) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    private Point translateToPopupMenu(Point point) {
        return this.translateToPopupMenu(point.x, point.y);
    }
    
    private Point translateToPopupMenu(int n, int y) {
        int x;
        if ((this.getParent() instanceof JPopupMenu)) {
            x = n - this.getSize().width;
        }
        else {
            x = n;
            y -= this.getSize().height;
        }
        return new Point(x, y);
    }
    
    public JPopupMenu getPopupMenu() {
        this.ensurePopupMenuCreated();
        return this.popupMenu;
    }
    
    public void addMenuListener(MenuListener l) {
        this.listenerList.add(MenuListener.class, l);
    }
    
    public void removeMenuListener(MenuListener l) {
        this.listenerList.remove(MenuListener.class, l);
    }
    
    public MenuListener[] getMenuListeners() {
        return (MenuListener[])this.listenerList.getListeners(MenuListener.class);
    }
    
    protected void fireMenuSelected() {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == MenuListener.class) {
                if (listenerList[i + 1] == null) {
                    throw new Error(this.getText() + " has a NULL Listener!! " + i);
                }
                if (this.menuEvent == null) {
                    this.menuEvent = new MenuEvent(this);
                }
                ((MenuListener)listenerList[i + 1]).menuSelected(this.menuEvent);
            }
        }
    }
    
    protected void fireMenuDeselected() {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == MenuListener.class) {
                if (listenerList[i + 1] == null) {
                    throw new Error(this.getText() + " has a NULL Listener!! " + i);
                }
                if (this.menuEvent == null) {
                    this.menuEvent = new MenuEvent(this);
                }
                ((MenuListener)listenerList[i + 1]).menuDeselected(this.menuEvent);
            }
        }
    }
    
    protected void fireMenuCanceled() {
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == MenuListener.class) {
                if (listenerList[i + 1] == null) {
                    throw new Error(this.getText() + " has a NULL Listener!! " + i);
                }
                if (this.menuEvent == null) {
                    this.menuEvent = new MenuEvent(this);
                }
                ((MenuListener)listenerList[i + 1]).menuCanceled(this.menuEvent);
            }
        }
    }
    
    @Override
    void configureAcceleratorFromAction(Action action) {
    }
    
    private ChangeListener createMenuChangeListener() {
        return new JMenu.MenuChangeListener();
    }
    
    protected JMenu.WinListener createWinListener(JPopupMenu popupMenu) {
        return new JMenu.WinListener(popupMenu);
    }
    
    @Override
    public void menuSelectionChanged(boolean selected) {
        this.setSelected(selected);
    }
    
    @Override
    public MenuElement[] getSubElements() {
        if (this.popupMenu == null) {
            return new MenuElement[0];
        }
        return new MenuElement[] { this.popupMenu };
    }
    
    @Override
    public Component getComponent() {
        return this;
    }
    
    @Override
    public void applyComponentOrientation(ComponentOrientation componentOrientation) {
        super.applyComponentOrientation(componentOrientation);
        if (this.popupMenu != null) {
            for (int menuComponentCount = this.getMenuComponentCount(), i = 0; i < menuComponentCount; ++i) {
                this.getMenuComponent(i).applyComponentOrientation(componentOrientation);
            }
            this.popupMenu.setComponentOrientation(componentOrientation);
        }
    }
    
    @Override
    public void setComponentOrientation(ComponentOrientation componentOrientation) {
        super.setComponentOrientation(componentOrientation);
        if (this.popupMenu != null) {
            this.popupMenu.setComponentOrientation(componentOrientation);
        }
    }
    
    @Override
    public void setAccelerator(KeyStroke keyStroke) {
        throw new Error("setAccelerator() is not defined for JMenu.  Use setMnemonic() instead.");
    }
    
    @Override
    protected void processKeyEvent(KeyEvent keyEvent) {
        MenuSelectionManager.defaultManager().processKeyEvent(keyEvent);
        if (keyEvent.isConsumed()) {
            return;
        }
        super.processKeyEvent(keyEvent);
        return;
    }
    
    @Override
    public void doClick(int n) {
        MenuSelectionManager.defaultManager().setSelectedPath(this.buildMenuElementArray(this));
    }
    
    private MenuElement[] buildMenuElementArray(JMenu menu) {
        Vector vector = new Vector();
        JPopupMenu popupMenu = menu.getPopupMenu();
        while (true) {
            if ((popupMenu instanceof JPopupMenu)) {
                JPopupMenu obj2 = (JPopupMenu)popupMenu;
                vector.insertElementAt(obj2, 0);
                popupMenu = obj2.getInvoker();
            }
            else if ((popupMenu instanceof JMenu)) {
                JMenu obj = (JMenu)popupMenu;
                vector.insertElementAt(obj, 0);
                popupMenu = obj.getParent();
            }
            else {
                if ((popupMenu instanceof JMenuBar)) {
                    break;
                }
                continue;
            }
        }
        vector.insertElementAt((JMenuBar)popupMenu, 0);
        MenuElement[] anArray = new MenuElement[vector.size()];
        vector.copyInto(anArray);
        return anArray;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.getUIClassID().equals("MenuUI")) {
            int n = (byte)(JComponent.getWriteObjCounter(this) - 1);
            JComponent.setWriteObjCounter(this, (byte)n);
            if (n == 0 && this.ui != null) {
                this.ui.installUI(this);
            }
        }
    }
    
    @Override
    protected String paramString() {
        return super.paramString();
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JMenu.AccessibleJMenu();
        }
        return this.accessibleContext;
    }
    
    static MenuElement[] access$000(JMenu menu, JMenu menu2) {
        return menu.buildMenuElementArray(menu2);
    }
    
    class MenuChangeListener implements ChangeListener, Serializable
    {
        boolean isSelected;
        
        MenuChangeListener() {
            this.isSelected = false;
        }
        
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            int selected = ((ButtonModel)changeEvent.getSource()).isSelected();
            if (selected != (this.isSelected ? 1 : 0)) {
                if (selected != 0) {
                    JMenu.this.fireMenuSelected();
                }
                else {
                    JMenu.this.fireMenuDeselected();
                }
                this.isSelected = selected;
            }
        }
    }
    
    protected class WinListener extends WindowAdapter implements Serializable
    {
        JPopupMenu popupMenu;
        
        public WinListener(JPopupMenu popupMenu) {
            this.popupMenu = popupMenu;
        }
        
        @Override
        public void windowClosing(WindowEvent windowEvent) {
            JMenu.this.setSelected(false);
        }
    }
    
    protected class AccessibleJMenu extends JMenuItem.AccessibleJMenuItem implements AccessibleSelection
    {
        @Override
        public int getAccessibleChildrenCount() {
            Component[] menuComponents = JMenu.this.getMenuComponents();
            int n = 0;
            for (int length = menuComponents.length, i = 0; i < length; ++i) {
                if ((menuComponents[i] instanceof Accessible)) {
                    ++n;
                }
            }
            return n;
        }
        
        @Override
        public Accessible getAccessibleChild(int n) {
            Component[] menuComponents = JMenu.this.getMenuComponents();
            int n2 = 0;
            for (Component component : menuComponents) {
                if ((component instanceof Accessible)) {
                    if (n2 == n) {
                        if ((component instanceof JComponent)) {
                            component.getAccessibleContext().setAccessibleParent(JMenu.this);
                        }
                        return (Accessible)component;
                    }
                    else {
                        ++n2;
                    }
                }
            }
            return null;
        }
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.MENU;
        }
        
        @Override
        public AccessibleSelection getAccessibleSelection() {
            return this;
        }
        
        @Override
        public int getAccessibleSelectionCount() {
            MenuElement[] selectedPath = MenuSelectionManager.defaultManager().getSelectedPath();
            if (selectedPath != null) {
                int i = 0;
                while (i < selectedPath.length) {
                    if (selectedPath[i] == JMenu.this && i + 1 < selectedPath.length) {
                        return 1;
                    }
                    ++i;
                }
            }
            return 0;
        }
        
        @Override
        public Accessible getAccessibleSelection(int n) {
            if (n < 0 || n >= JMenu.this.getItemCount()) {
                return null;
            }
            MenuElement[] selectedPath = MenuSelectionManager.defaultManager().getSelectedPath();
            if (selectedPath != null) {
                for (int i = 0; i < selectedPath.length; ++i) {
                    if (selectedPath[i] == JMenu.this) {
                        while (++i < selectedPath.length) {
                            if (!(selectedPath[i] instanceof JMenuItem)) {
                                continue;
                            }
                            return (Accessible)selectedPath[i];
                        }
                    }
                }
            }
            return null;
        }
        
        @Override
        public boolean isAccessibleChildSelected(int n) {
            MenuElement[] selectedPath = MenuSelectionManager.defaultManager().getSelectedPath();
            if (selectedPath != null) {
                JMenuItem item = JMenu.this.getItem(n);
                int i = 0;
                while (i < selectedPath.length) {
                    if (selectedPath[i] == item) {
                        return true;
                    }
                    ++i;
                }
            }
            return false;
        }
        
        @Override
        public void addAccessibleSelection(int n) {
            if (n < 0 || n >= JMenu.this.getItemCount()) {
                return;
            }
            JMenuItem item = JMenu.this.getItem(n);
            if (item != null) {
                if ((item instanceof JMenu)) {
                    MenuSelectionManager.defaultManager().setSelectedPath(JMenu.access$000(JMenu.this, (JMenu)item));
                }
                else {
                    MenuSelectionManager.defaultManager().setSelectedPath((MenuElement[])null);
                }
            }
            return;
        }
        
        @Override
        public void removeAccessibleSelection(int n) {
            if (n < 0 || n >= JMenu.this.getItemCount()) {
                return;
            }
            JMenuItem item = JMenu.this.getItem(n);
            if (item != null && item instanceof JMenu && item.isSelected()) {
                MenuElement[] selectedPath = MenuSelectionManager.defaultManager().getSelectedPath();
                MenuElement[] selectedPath2 = new MenuElement[selectedPath.length - 2];
                for (int i = 0; i < selectedPath.length - 2; ++i) {
                    selectedPath2[i] = selectedPath[i];
                }
                MenuSelectionManager.defaultManager().setSelectedPath(selectedPath2);
            }
            return;
        }
        
        @Override
        public void clearAccessibleSelection() {
            MenuElement[] selectedPath = MenuSelectionManager.defaultManager().getSelectedPath();
            if (selectedPath != null) {
                for (int i = 0; i < selectedPath.length; ++i) {
                    if (selectedPath[i] == JMenu.this) {
                        MenuElement[] selectedPath2 = new MenuElement[i + 1];
                        System.arraycopy(selectedPath, 0, selectedPath2, 0, i);
                        selectedPath2[i] = JMenu.this.getPopupMenu();
                        MenuSelectionManager.defaultManager().setSelectedPath(selectedPath2);
                    }
                }
            }
        }
        
        @Override
        public void selectAllAccessibleSelection() {
        }
    }
}
